
import boto3


def ec2_instances():
    """
    SQL Usage:
    SELECT * FROM python_table('aws', 'ec2_instances',
      {
        'instance_id': 'VARCHAR',
        'name': 'VARCHAR',
        'instance_type': 'VARCHAR',
        'state': 'VARCHAR',
        'key_pair': 'VARCHAR',
        'platform': 'VARCHAR',
        'architecture': 'VARCHAR',
        'vpc_id': 'VARCHAR',
        'subnet_id': 'VARCHAR',
        'public_dns': 'VARCHAR',
        'public_ip': 'VARCHAR',
        'private_dns': 'VARCHAR',
        'private_ip': 'VARCHAR'
        }, []);
    """
    def response_to_rows(response):
        for resv in response['Reservations']:
            resv_id = resv['ReservationId']
            instances = resv['Instances']
            for i in instances:
                instance_name = None
                for pair in i.get('Tags', []):
                    if pair['Key'] == 'Name':
                        instance_name = pair['Value']
                        break
                yield (
                    i['InstanceId'],
                    instance_name,
                    i['InstanceType'],
                    i['State']['Name'],
                    i['KeyName'],
                    # i['LaunchTime'].strftime('%m/%d/%Y'),
                    i.get('Platform'),
                    i['Architecture'],
                    i['VpcId'],
                    i['SubnetId'],
                    i['PublicDnsName'],
                    i['PublicIpAddress'],
                    i['PrivateDnsName'],
                    i['PrivateIpAddress'],
                    # i['HibernationOptions']['Configured'],
                    )
    client = boto3.client('ec2')
    response = client.describe_instances()
    yield from response_to_rows(response)
    token = response.get('NextToken')
    while token:
        response = client.describe_instances(NextToken = token)
        yield from response_to_rows(response)
        token = response.get('NextToken')
        
        

def s3_buckets():
    """
    SQL Usage:
    SELECT * FROM python_table('aws', 's3_buckets', {'name': 'VARCHAR', 'creation_date': 'VARCHAR'}, []);
    """
    client = boto3.client('s3')
    response = client.list_buckets()
    for bucket in response['Buckets']:
        yield (bucket['Name'], bucket['CreationDate'].strftime('%m/%d/%Y'))
    

def s3_objects(bucket, prefix = None):
    """
    SQL Usage:
    SELECT * FROM python_table('aws', 's3_objects',
      { 'key': 'VARCHAR', 'last_modified': 'VARCHAR', 'size': 'INT', 'storage_class': 'VARCHAR'},
      ['bucket-name', 'foo/bar/prefix']);

    Note that the final prefix argument is optional, and you don't need to specify it in
    your SQL query. To list all objects:
    SELECT * FROM python_table('aws', 's3_objects',
      { 'key': 'VARCHAR', 'last_modified': 'VARCHAR', 'size': 'INT', 'storage_class': 'VARCHAR'},
      ['bucket-name']);
    """
    def to_row(obj):
        return (obj['Key'],
                obj['LastModified'].strftime('%m/%d/%Y'),
                obj['Size'],
                obj['StorageClass'])

    # Note that the list_objects_v2 doesn't let you specify a meaningless
    # Prefix value. So we can't just pass in a None. We have to check if
    # a value was specified, and only use it as an argument if one exists.
    kwargs = {
        'Bucket': bucket
        }
    if prefix:
        kwargs['Prefix']=prefix

        
    client = boto3.client('s3')
    response = client.list_objects_v2(**kwargs)
    for obj in response.get('Contents', []):
        yield to_row(obj)

    while response['IsTruncated']:
        # Continue to make additional requests, this happens because each
        # list objects request is capped at 1000 responses.
        kwargs['ContinuationToken'] = response['NextContinuationToken']
        response = client.list_objects_v2(**kwargs)
        for obj in response['Contents']:
            yield to_row(obj)
        



# Note this file is named `google_` instead of `google` to prevent name conflicts
# with official google libraries that would prevent this script from being importable.

import os, sys
import gspread
from oauth2client.service_account import ServiceAccountCredentials
from datetime import datetime
from googleapiclient.discovery import build
from google.oauth2 import service_account

def _key_path(key_file_path = None):
    if not key_file_path:
        key_file_path = os.environ.get('GOOGLE_APPLICATION_CREDENTIALS')
        if not key_file_path:
            raise Exception("No key file specified, and no environment variable found")
    return key_file_path
    

def sheet(sheets_url_or_key, cell_range, key_file_path = None):
    """
    SQL Usage:
    SELECT *
    FROM pytable('google_:sheet', '1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms', 'Class Data!A2:F31',
         columns = {
             'name': 'VARCHAR', 'gender': 'VARCHAR', 'class_level': 'VARCHAR',
             'state': 'VARCHAR', 'major': 'VARCHAR', 'extracurricular': 'VARCHAR'}
    );
    """
    key_file_path = _key_path(key_file_path)

    # Support specifying the full ULR, or the shorter 'key' identierifer. We generate
    # the full url if the later is supplied.
    if sheets_url_or_key.startswith('https://docs.google.com/spreadsheets/d/'):
        sheets_url = sheets_url_or_key
    else:
        sheets_url = "https://docs.google.com/spreadsheets/d/" + sheets_url_or_key + "/edit#gid=0"
        
    # authorize access to the Google Sheets API
    scope = ['https://spreadsheets.google.com/feeds',
             'https://www.googleapis.com/auth/drive']
    creds = ServiceAccountCredentials.from_json_keyfile_name(key_file_path, scope)
    client = gspread.authorize(creds)

    # extract the worksheet name and range from the cell range string
    worksheet_name, cell_range = cell_range.split('!')
    worksheet = client.open_by_url(sheets_url).worksheet(worksheet_name)

    # get the values of the specified cells and convert to Python types
    cell_list = worksheet.range(cell_range)
    # rows = []
    current_row = []
    for cell in cell_list:
        if cell.value.isdigit():
            current_row.append(int(cell.value))
        elif cell.value.replace('.', '', 1).isdigit():
            current_row.append(float(cell.value))
        elif cell.value.startswith('='):
            # evaluate formulas and include result in output
            formula_value = worksheet.cell(cell.row, cell.col).value
            try:
                formula_result = worksheet.cell(cell.row, cell.col).value
                if isinstance(formula_result, (int, float)):
                    current_row.append(formula_result)
                elif isinstance(formula_result, datetime):
                    current_row.append(formula_result)
                else:
                    current_row.append(formula_value)
            except:
                current_row.append(formula_value)
        else:
            try:
                # try to parse date and time values
                date_time = datetime.strptime(cell.value, '%m/%d/%Y %H:%M:%S')
                current_row.append(date_time)
            except ValueError:
                current_row.append(cell.value)
        # check if we've reached the end of a row
        if cell.col == ord('F') - ord('A') + 1:
            yield current_row
            current_row = []
    # return rows

if __name__ == '__main__':
    sheet_url = "https://docs.google.com/spreadsheets/d/1BxiMVs0XRA5nFMdKvBdBZjgmUUqptlbs74OgvE2upms/edit#gid=0"
    cell_range = "Class Data!A2:F31"
    rows = sheet(sheet_url, cell_range)
    # for r in rows:
    #     print(r)

    view_id = '...........'
    start_date = '2023-01-01'
    end_date = '2023-03-31'

    data_iterator = analytics(view_id, start_date, end_date)

    # Iterate over the data
    for record in data_iterator:
        print(record)






import os, sys
import openai

openai.organization = os.getenv("OPENAI_ORG_ID")
openai.api_key = os.getenv("OPENAI_API_KEY")

def prompt(input_phrase, num_responses = 5):
    """
    SQL Usage:
    SELECT * FROM pytable('chatgpt', 'prompt',
      {'index': 'INT','message': 'VARCHAR', 'message_role': 'VARCHAR', 'finish_reason': 'VARCHAR'},
      ['Write a limerick poem about how much you love SQL', 2]);
    """
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        # Complains when getting int like things so we explicitly cast
        n = int(num_responses),
        messages=[
            {"role": "user", "content": input_phrase}
            ]
        )
    for choice in completion.choices:
        yield(
            choice['index'],
            choice['message']['content'],
            choice['message']['role'],
            choice['finish_reason'],
            )

def main(args):
    for msg in prompt("Hello there!"):
        print(msg)

if __name__ == '__main__':
    sys.exit(main(sys.argv))




from googleapiclient.discovery import build
from google.oauth2 import service_account
from ducktables.google import _key_path

def metrics(view_id, start_date, end_date, dimensions = None, metrics = None, key_file_path = None):
    """
    SQL Usage:
    SELECT * FROM pytable('ducktables.google.analytics:metrics', '<view-id>', '<start-date>', '<end-date>',
      columns = {
        -- Dimensions
        'date': 'VARCHAR', 'hour': 'VARCHAR', 'pagePath': 'VARCHAR', 'source': 'VARCHAR', 'medium': 'VARCHAR',
        -- Metrics
        'sessions': 'INT', 'users': 'INT', 'pageviews': 'INT', 'avgSessionDuration': 'FLOAT', 'bounceRate': 'FLOAT'
        });

    # Note that this function lets you specify a list of dimensions and/or metrics. If you use non-default
    # values for these you will need to update the 'columns' struct from the example above.
    """
    key_file_path = _key_path(key_file_path)
    # Load credentials from JSON file
    credentials = service_account.Credentials.from_service_account_file(key_file_path)
    
    # Build the Google Analytics Reporting API client
    analytics = build('analyticsreporting', 'v4', credentials=credentials)

    if not metrics:
        metrics = ['ga:sessions', 'ga:users', 'ga:pageviews', 'ga:avgSessionDuration', 'ga:bounceRate']
    elif isinstance(metrics, str):
        metrics = [metrics]
    if not dimensions:
        dimensions = ['ga:date', 'ga:hour', 'ga:pagePath', 'ga:source', 'ga:medium']
    elif isinstance(dimensions, str):
        dimensions = [dimensions]
    
    # Retrieve GA data using the API
    response = analytics.reports().batchGet(
        body={
            'reportRequests': [{
                'viewId': view_id,
                'dateRanges': [{'startDate': start_date, 'endDate': end_date}],
                # 'metrics': [{'expression': 'ga:sessions'}, {'expression': 'ga:users'}],
                'metrics': [ {'expression': m} for m in metrics],
                # 'dimensions': [{'name': 'ga:date'}]
                'dimensions': [{'name': d} for d in dimensions],
            }]
        }
    ).execute()
    
    # Extract data from the API response
    report = response['reports'][0]
    for row in report['data']['rows']:
        # All values are returned as strings because sure why not?
        # So we iterate over each returned value and use a heuristic
        # of if there's a '.' char its a float, otherwise its an int.
        values = row['metrics'][0]['values']
        coerced = []
        for v in values:
            if '.' in v:
                coerced.append(float(v))
            else:
                coerced.append(int(v))
        yield row['dimensions'] + coerced


def events(view_id, start_date, end_date, key_file_path = None):
    """
    SQL Usage:

    SELECT * FROM pytable('ducktables.google.analytics:events', '<view-id>', '<start-date>', '<end-date>',
      columns = {
        -- Dimensions
        'date': 'VARCHAR', 'hour': 'VARCHAR', 'category': 'VARCHAR', 'action': 'VARCHAR', 'label': 'VARCHAR',
        'pagePath': 'VARCHAR', 'source': 'VARCHAR', 'medium': 'VARCHAR',

        -- Metrics
        'event_count': 'INT'
        });
    """
    dimension_names = [
        'ga:date', 'ga:hour', 'ga:eventCategory', 'ga:eventAction',
        'ga:eventLabel', 'ga:pagePath', 'ga:source', 'ga:medium'
        ]
    metric_names = ['ga:totalEvents']
    return metrics(view_id, start_date, end_date, dimension_names, metric_names, key_file_path)





import os, sys
from github import Github

token = os.environ.get("GITHUB_ACCESS_TOKEN")
g = Github(token)

def repos_for(username):
    for r in g.get_user(username).get_repos():
        yield (r.name, r.description, r.language)

def main(args):
    for r in repos_for('markroddy'):
        print(r)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



from setuptools import setup, find_packages

setup(
    name='ducktables',
    version='0.1.2',
    description='Python Table Functions for DuckDB',
    packages=find_packages(),
    install_requires=open('requirements.txt').read().splitlines(),
    )
